'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var prettyFormat = require('pretty-format');
var isHtml = require('is-html');

var _require = require('./replace-selectors'),
    replaceSelectors = _require.replaceSelectors;

var _require2 = require('./utils'),
    getNodes = _require2.getNodes,
    markNodes = _require2.markNodes,
    getSelectors = _require2.getSelectors,
    getStylesAndAllSelectors = _require2.getStylesAndAllSelectors;

// doing this to make it easier for users to mock things
// like switching between development mode and whatnot.


var getGlamorStyleSheet = function () {
  return require('glamor').styleSheet;
};

function createSerializer(styleSheet) {
  // eslint-disable-next-line complexity
  return { test: function (val) {
      if (!val) {
        return false;
      }
      return isHtmlStringWithGlamorClasses(val) || !val.serializedWithJestGlamorReact && (val.$$typeof === Symbol.for('react.test.json') || val instanceof HTMLElement && !isBeingSerialized(val));
    }, print: function (val, printer) {
      if (typeof val === 'string') {
        return fromHTMLString(val);
      }
      var nodes = getNodes(val);
      markNodes(nodes);
      var selectors = getSelectors(nodes);

      var _getStylesAndAllSelec = getStylesAndAllSelectors(selectors, styleSheet),
          styles = _getStylesAndAllSelec.styles,
          allSelectors = _getStylesAndAllSelec.allSelectors;

      var printedVal = printer(val);
      // allows us to take multiple snapshots
      val.serializedWithJestGlamorReact = false;
      if (styles) {
        return replaceSelectors(allSelectors, styles, printedVal);
      } else {
        return printedVal;
      }
    } };
}

function isBeingSerialized(node) {
  var currentNode = node;

  while (currentNode) {
    if (currentNode.serializedWithJestGlamorReact) {
      return true;
    }
    currentNode = currentNode.parentNode;
  }
  return false;
}

function isHtmlStringWithGlamorClasses(string) {
  // is-html was letting a "Snapshot Diff:" get through
  return typeof string === 'string' && string.startsWith('<') && isHtml(string) && string.includes('css-');
}

function fromDOMNode(div) {
  var extract = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function (r) {
    return r.children[0];
  };

  var root = document.createElement('div');
  // not appending because we don't want to mess up the div they give us
  root.innerHTML = div.outerHTML;
  var nodes = getNodes(root);
  var selectors = getSelectors(nodes);

  var _getStylesAndAllSelec2 = getStylesAndAllSelectors(selectors, getGlamorStyleSheet()),
      styles = _getStylesAndAllSelec2.styles,
      allSelectors = _getStylesAndAllSelec2.allSelectors;

  return replaceSelectors(allSelectors, styles, prettyFormat(extract(root), {
    plugins: [prettyFormat.plugins.DOMElement, prettyFormat.plugins.DOMCollection]
  }));
}

function fromHTMLString(string) {
  var div = document.createElement('div');
  div.innerHTML = string;
  return fromDOMNode(div, function (root) {
    return root.children[0].children[0];
  });
}

module.exports = Object.assign(createSerializer, _extends({}, createSerializer(getGlamorStyleSheet), {
  fromHTMLString,
  fromDOMNode
}));